#!/usr/bin/perl
#
# atxtowin.pm ############################################################################
#
# Conversion from ArabTeX ASCII transliteration to Windows CP 1256
#
# 2000/11/13 by Otakar Smrz, Charles University, Prague ##################################

# This Perl module should interpret the Arabic transliteration in the way ArabTeX does it,
# as far as we consider conventional everyday requirements. Its principal use might rest
# in generating Arabic HTML documents, still with no need for localized software.
#
# When calling the conversion subroutine, one must feed in two arguments: the text itself
# and a reference to a hash of conversion parameters, keys and values of which can comply
# with user's wishes if redefined in the lower parts of the code.
#
# Enjoy the ride and let me know of all possible improvements!

package Atxtowin;

use Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(&atxtowin);

sub atxtowin
{
 my($text)="\n".$_[0]."\n";
 my(%par)=%{$_[1]};

 my(%ArabTeX)=(
                'p',          "\201",    # 81
                '\^c',        "\215",    # 8D
                '\^z',        "\216",    # 8E
                'g',          "\220",    # 90
                '\,',         "\241",    # A1
                '\;',         "\272",    # BA
                '\?',         "\277",    # BF
                '\\\\H{}',    "\301",    # C1
                '\\\\M{}',    "\302",    # C2
                '\\\\H{a}',   "\303",    # C3
                '\\\\H{w}',   "\304",    # C4
                '\\\\H{i}',   "\305",    # C5
                '\\\\H{y}',   "\306",    # C6
                '\\\\L{}',    "\307",    # C7
                'b',          "\310",    # C8
                'T',          "\311",    # C9
                't',          "\312",    # CA
                '\_t',        "\313",    # CB
                '\^g',        "\314",    # CC
                '\.h',        "\315",    # CD
                '\_h',        "\316",    # CE
                'd',          "\317",    # CF
                '\_d',        "\320",    # D0
                'r',          "\321",    # D1
                'z',          "\322",    # D2
                's',          "\323",    # D3
                '\^s',        "\324",    # D4
                '\.s',        "\325",    # D5
                '\.d',        "\326",    # D6
                '\.t',        "\330",    # D8
                '\.z',        "\331",    # D9
                '\`',         "\332",    # DA
                '\.g',        "\333",    # DB
                'f',          "\335",    # DD
                'q',          "\336",    # DE
                'k',          "\337",    # DF
                'l',          "\341",    # E1
                'm',          "\343",    # E3
                'n',          "\344",    # E4
                'h',          "\345",    # E5
                'w',          "\346",    # E6
                '\\\\Q{}',    "\354",    # EC
                'y',          "\355",    # ED
                '\\\\V{aN}',  "\360",    # F0
                '\\\\V{uN}',  "\361",    # F1
                '\\\\V{iN}',  "\362",    # F2
                '\\\\V{a}',   "\363",    # F3
                '\\\\V{u}',   "\365",    # F5
                '\\\\V{i}',   "\366",    # F6
                '\\\\D{}',    "\370",    # F8
                '\\\\O{}',    "\372",    # FA
              );

 my(@shams)=('t','\_t','d','\_d','r','z','s','\^s','\.s','\.d','\.t','\.z','l','n');
 my(@qamar)=('b','\^g','\.h','\_h','\`','\.g','f','q','k','m','h','w','y');
 my($one);

 $text=~s/_a/a/g; # is there any special character for the defective fatha?
 $text=~s/_U/U/g; # _U only affects the pronunciation, not the spelling

 foreach $one (@shams,'\\\'',@qamar)
 {
  $text=~s/([^\_\.\^]$one)([^${one}aiuAIUY\-T])/$1\\O{}$2/g;
  $text=~s/([^\_\.\^])($one)$one/$1$2\\D{}/g;
  # alas! using $one in the replacing expression produces extra \\
 }

 $text=~s/([\s\-])([aiu])/$1\\L{}$2/gs;

 $text=~s/([\s\-])'([auUY])/$1\\H{a}$2/gs;
 $text=~s/([\s\-])'([iI])/$1\\H{i}$2/gs;
 $text=~s/([\s\-])'A/$1\\M{}/gs;

 $text=~s/(I|y\\O{})'(aN[AY]?[\s\-])/$1\\H{y}$2/gs;
 $text=~s/a'(|\\D{})(|[au]N?)([\s\-])/a\\H{a}$1$2$3/gs;
 $text=~s/a'(|\\D{})(iN?)([\s\-])/a\\H{i}$1$2$3/gs;
 $text=~s/i'(|\\D{})(|[aiu]N?)([\s\-])/i\\H{y}$1$2$3/gs;
 $text=~s/u'(|\\D{})(|[aiu]N?)([\s\-])/u\\H{w}$1$2$3/gs;
 $text=~s/'(|\\D{})(|[aiu]N?)([\s\-])/\\H{}$1$2$3/gs;

 $text=~s/([iI])'(|\\D{})/$1\\H{y}$2/g;
 $text=~s/'(|\\D{})([iI])/\\H{y}$1$2/g;
 $text=~s/([uU])'(|\\D{})/$1\\H{w}$2/g;
 $text=~s/'(|\\D{})([uU])/\\H{w}$1$2/g;
 $text=~s/A'(|\\D{})/A\\H{}$1/g; # how do you write <rA''AsuN>?
 $text=~s/a'(\\D{})/a\\H{a}$1/g; # how do you write <ra''AsuN>?
 $text=~s/'A/\\M{}/g;
 $text=~s/'a/\\H{a}a/g;

 foreach $one (@shams) { $text=~s/($one)\-$one/l$1\\D{}/g; }
 foreach $one ('\\\\',@qamar) { $text=~s/l\-($one)/l\\O{}$1/g; }
 # alas! using $one in the replacing expression produces extra \\

 $text=~s/aNY/\\V{aN}\\Q{}/g;
 $text=~s/(A\\H{}|\\H{a})aN/$1\\V{aN}/g;
 $text=~s/([^{])aNA?/$1\\V{aN}\\L{}/g;
 $text=~s/([^{])iN/$1\\V{iN}/g;
 $text=~s/([^{])uN/$1\\V{uN}/g;

 $text=~s/UA/\\V{u}w\\L{}/g;
 $text=~s/aWA?/\\V{a}w\\O{}\\L{}/g;

 $text=~s/([^{])a/$1\\V{a}/g;
 $text=~s/([^{])i/$1\\V{i}/g;
 $text=~s/([^{])u/$1\\V{u}/g;

 $text=~s/A/\\V{a}\\L{}/g;
 $text=~s/Iy/\\V{i}y\\D{}/g;
 $text=~s/I/\\V{i}y/g;
 $text=~s/Uw/\\V{u}w\\D{}/g;
 $text=~s/U/\\V{u}w/g;
 $text=~s/Y/\\V{a}\\Q{}/g;

 $text=~s/\-//g;

 $text=~s/l((\\V\{[aiu]\}|\\D\{\}|\\O\{\})+)(\\L\{\}|\\H\{[ai]\}|\\M\{\})/l$3$1/g;

 if (defined($par{"vocalize"}))
 {
  if ($par{"vocalize"} eq "full") { }
  else
  {
   $text=~s/\\O{}//g;
   if ($par{"vocalize"} eq "vowels") { }
   else
   {
    $text=~s/\\V{[aiu]N?}//g;
    if ($par{"vocalize"} eq "shadda") { }
    else
    {
     $text=~s/\\D{}//g;
     if ($par{"vocalize"} eq "none") { $text=~s/\\D{}//g; }
    }
   }
  }
 }


 foreach $one (keys(%ArabTeX))
 {
  while ($text=~s/([^\_\.\^])$one/$1$ArabTeX{$one}/) { } # /g does not generally work well
 }


 if (defined($par{"separate"}))
 {
  if ($par{"separate"} eq "no") { $text=substr($text,1,-1); } # remove additional "\n"s
 }

 return $text;
}

1;
